@@ -1,24 +1,14 @@
 # cspell:ignore einsum
-# pylint: disable=arguments-differ,no-member,protected-access,too-many-lines
-# pylint: disable=unused-argument
+# pylint: disable=arguments-differ, no-member, protected-access, too-many-lines
+# pylint: disable=unused-argument, W0223
+# https://stackoverflow.com/a/22224042
 """Classes and functions for relativistic four-momentum kinematics."""
+from __future__ import annotations
 
 import itertools
-import sys
 from collections import abc
 from functools import singledispatch
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    FrozenSet,
-    Iterable,
-    List,
-    Optional,
-    Sequence,
-    Set,
-    Union,
-)
+from typing import Dict, Iterable, Mapping
 
 import attrs
 import sympy as sp
@@ -57,12 +47,6 @@ from ampform.sympy._array_expressions import (
 )
 from ampform.sympy.math import ComplexSqrt
 
-if TYPE_CHECKING:  # pragma: no cover
-    if sys.version_info < (3, 10):
-        from typing_extensions import TypeAlias
-    else:
-        from typing import TypeAlias
-
 
 class HelicityAdapter:
     r"""Converter for four-momenta to kinematic variable data.
@@ -76,9 +60,7 @@ class HelicityAdapter:
 
     def __init__(
         self,
-        transitions: Union[
-            ReactionInfo, Iterable[Union[Topology, StateTransition]]
-        ],
+        transitions: (ReactionInfo | Iterable[Topology | StateTransition]),
     ) -> None:
         self.__topologies = _extract_topologies(transitions)
         for topology in self.__topologies:
@@ -103,7 +85,7 @@ class HelicityAdapter:
         self.__topologies.add(topology)
 
     @property
-    def registered_topologies(self) -> FrozenSet[Topology]:
+    def registered_topologies(self) -> frozenset[Topology]:
         return frozenset(self.__topologies)
 
     def permutate_registered_topologies(self) -> None:
@@ -126,7 +108,7 @@ class HelicityAdapter:
 
     def create_expressions(
         self, generate_wigner_angles: bool = False
-    ) -> Dict[str, sp.Expr]:
+    ) -> dict[str, sp.Expr]:
         output = {}
         for topology in self.__topologies:
             momenta = create_four_momentum_symbols(topology)
@@ -146,23 +128,23 @@ class HelicityAdapter:
 
 @singledispatch
 def _extract_topologies(
-    obj: Union[ReactionInfo, Iterable[Union[Topology, StateTransition]]]
-) -> Set[Topology]:
+    obj: ReactionInfo | Iterable[Topology | StateTransition],
+) -> set[Topology]:
     raise TypeError(f"Cannot extract topologies from a {type(obj).__name__}")
 
 
 @_extract_topologies.register(ReactionInfo)
-def _(transitions: ReactionInfo) -> Set[Topology]:
+def _(transitions: ReactionInfo) -> set[Topology]:
     return _extract_topologies(transitions.transitions)
 
 
 @_extract_topologies.register(abc.Iterable)
-def _(transitions: abc.Iterable) -> Set[Topology]:
+def _(transitions: abc.Iterable) -> set[Topology]:
     return {_get_topology(t) for t in transitions}
 
 
 @singledispatch
-def _get_topology(obj: Any) -> Topology:
+def _get_topology(obj) -> Topology:
     raise TypeError(
         f"Cannot create a {Topology.__name__} from a {type(obj).__name__}"
     )
@@ -178,7 +160,7 @@ def _(obj: StateTransition) -> Topology:
     return obj.topology
 
 
-def create_four_momentum_symbols(topology: Topology) -> "FourMomenta":
+def create_four_momentum_symbols(topology: Topology) -> FourMomenta:
     """Create a set of array-symbols for a `~qrules.topology.Topology`.
 
     >>> from qrules.topology import create_isobar_topologies
@@ -187,7 +169,9 @@ def create_four_momentum_symbols(topology: Topology) -> "FourMomenta":
     {0: p0, 1: p1, 2: p2}
     """
     n_final_states = len(topology.outgoing_edge_ids)
-    return {i: FourMomentumSymbol(f"p{i}") for i in range(n_final_states)}
+    return {
+        i: FourMomentumSymbol(f"p{i}", shape=[]) for i in range(n_final_states)
+    }
 
 
 FourMomenta = Dict[int, "FourMomentumSymbol"]
@@ -196,7 +180,7 @@ FourMomenta = Dict[int, "FourMomentumSymbol"]
 It's best to create a `dict` of `FourMomenta` with
 :func:`create_four_momentum_symbols`.
 """
-FourMomentumSymbol: "TypeAlias" = ArraySymbol
+FourMomentumSymbol = ArraySymbol
 r"""Array-`~sympy.core.symbol.Symbol` that represents an array of four-momenta.
 
 The array is assumed to be of shape :math:`n\times 4` with :math:`n` the number
@@ -215,17 +199,17 @@ ArraySlice = make_commutative(ArraySlice)  # type: ignore[misc]
 class Energy(UnevaluatedExpression):
     """Represents the energy-component of a `FourMomentumSymbol`."""
 
-    def __new__(cls, momentum: "FourMomentumSymbol", **hints: Any) -> "Energy":
+    def __new__(cls, momentum: sp.Basic, **hints) -> Energy:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> ArraySlice:
         return ArraySlice(self._momentum, (slice(None), 0))
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self._momentum)
         return Rf"E\left({momentum}\right)"
 
@@ -236,14 +220,12 @@ class Energy(UnevaluatedExpression):
 class FourMomentumX(UnevaluatedExpression):
     """Component :math:`x` of a `FourMomentumSymbol`."""
 
-    def __new__(
-        cls, momentum: "FourMomentumSymbol", **hints: Any
-    ) -> "FourMomentumX":
+    def __new__(cls, momentum: sp.Basic, **hints) -> FourMomentumX:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> ArraySlice:
         return ArraySlice(self._momentum, (slice(None), 1))
@@ -255,14 +237,12 @@ class FourMomentumX(UnevaluatedExpression):
 class FourMomentumY(UnevaluatedExpression):
     """Component :math:`y` of a `FourMomentumSymbol`."""
 
-    def __new__(
-        cls, momentum: "FourMomentumSymbol", **hints: Any
-    ) -> "FourMomentumY":
+    def __new__(cls, momentum: sp.Basic, **hints) -> FourMomentumY:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> ArraySlice:
         return ArraySlice(self._momentum, (slice(None), 2))
@@ -274,14 +254,12 @@ class FourMomentumY(UnevaluatedExpression):
 class FourMomentumZ(UnevaluatedExpression):
     """Component :math:`z` of a `FourMomentumSymbol`."""
 
-    def __new__(
-        cls, momentum: "FourMomentumSymbol", **hints: Any
-    ) -> "FourMomentumZ":
+    def __new__(cls, momentum: sp.Basic, **hints) -> FourMomentumZ:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> ArraySlice:
         return ArraySlice(self._momentum, (slice(None), 3))
@@ -289,17 +267,15 @@ class FourMomentumZ(UnevaluatedExpression):
 
 @implement_doit_method
 @make_commutative
-class ThreeMomentum(UnevaluatedExpression):
+class ThreeMomentum(UnevaluatedExpression, NumPyPrintable):
     """Spatial components of a `FourMomentumSymbol`."""
 
-    def __new__(
-        cls, momentum: "FourMomentumSymbol", **hints: Any
-    ) -> "ThreeMomentum":
+    def __new__(cls, momentum: sp.Basic, **hints) -> ThreeMomentum:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> ArraySlice:
         three_momentum = ArraySlice(
@@ -307,36 +283,34 @@ class ThreeMomentum(UnevaluatedExpression):
         )
         return three_momentum
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self._momentum)
         return Rf"\vec{{{momentum}}}"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         return printer._print(self.evaluate())
 
 
 @implement_doit_method
 @make_commutative
-class EuclideanNorm(UnevaluatedExpression):
+class EuclideanNorm(UnevaluatedExpression, NumPyPrintable):
     """Take the euclidean norm of an array over axis 1."""
 
-    def __new__(
-        cls, vector: "FourMomentumSymbol", **hints: Any
-    ) -> "EuclideanNorm":
+    def __new__(cls, vector: sp.Basic, **hints) -> EuclideanNorm:
         return create_expression(cls, vector, **hints)
 
     @property
-    def _vector(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _vector(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> ArraySlice:
         return sp.sqrt(EuclideanNormSquared(self._vector))
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         vector = printer._print(self._vector)
         return Rf"\left|{vector}\right|"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         return printer._print(self.evaluate())
 
 
@@ -345,27 +319,25 @@ class EuclideanNorm(UnevaluatedExpression):
 class EuclideanNormSquared(UnevaluatedExpression):
     """Take the squared euclidean norm of an array over axis 1."""
 
-    def __new__(
-        cls, vector: "FourMomentumSymbol", **hints: Any
-    ) -> "EuclideanNormSquared":
+    def __new__(cls, vector: sp.Basic, **hints) -> EuclideanNormSquared:
         return create_expression(cls, vector, **hints)
 
     @property
-    def _vector(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _vector(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
-    def evaluate(self) -> ArraySlice:
+    def evaluate(self) -> ArrayAxisSum:
         return ArrayAxisSum(self._vector**2, axis=1)
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         vector = printer._print(self._vector)
         return Rf"\left|{vector}\right|^{{2}}"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         return printer._print(self.evaluate())
 
 
-def three_momentum_norm(momentum: FourMomentumSymbol) -> EuclideanNorm:
+def three_momentum_norm(momentum: sp.Basic) -> EuclideanNorm:
     return EuclideanNorm(ThreeMomentum(momentum))
 
 
@@ -374,19 +346,19 @@ def three_momentum_norm(momentum: FourMomentumSymbol) -> EuclideanNorm:
 class InvariantMass(UnevaluatedExpression):
     """Invariant mass of a `FourMomentumSymbol`."""
 
-    def __new__(cls, momentum: "FourMomentumSymbol", **hints: Any) -> "Energy":
+    def __new__(cls, momentum: sp.Basic, **hints) -> InvariantMass:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
-    def evaluate(self) -> ArraySlice:
+    def evaluate(self) -> ComplexSqrt:
         p = self._momentum
         p_xyz = ThreeMomentum(p)
         return ComplexSqrt(Energy(p) ** 2 - EuclideanNorm(p_xyz) ** 2)
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self._momentum)
         return f"m_{{{momentum}}}"
 
@@ -396,18 +368,18 @@ class InvariantMass(UnevaluatedExpression):
 class Phi(UnevaluatedExpression):
     r"""Azimuthal angle :math:`\phi` of a `FourMomentumSymbol`."""
 
-    def __new__(cls, momentum: "FourMomentumSymbol", **hints: Any) -> "Phi":
+    def __new__(cls, momentum: sp.Basic, **hints) -> Phi:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> sp.Expr:
         p = self._momentum
         return sp.atan2(FourMomentumY(p), FourMomentumX(p))
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self._momentum)
         return Rf"\phi\left({momentum}\right)"
 
@@ -417,18 +389,18 @@ class Phi(UnevaluatedExpression):
 class Theta(UnevaluatedExpression):
     r"""Polar (elevation) angle :math:`\theta` of a `FourMomentumSymbol`."""
 
-    def __new__(cls, momentum: "FourMomentumSymbol", **hints: Any) -> "Theta":
+    def __new__(cls, momentum: sp.Basic, **hints) -> Theta:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> sp.Expr:
         p = self._momentum
         return sp.acos(FourMomentumZ(p) / three_momentum_norm(p))
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self._momentum)
         return Rf"\theta\left({momentum}\right)"
 
@@ -438,19 +410,19 @@ class Theta(UnevaluatedExpression):
 class NegativeMomentum(UnevaluatedExpression):
     r"""Invert the spatial components of a `FourMomentumSymbol`."""
 
-    def __new__(cls, momentum: "FourMomentumSymbol", **hints: Any) -> "Theta":
+    def __new__(cls, momentum: sp.Basic, **hints) -> NegativeMomentum:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "FourMomentumSymbol":
-        return self.args[0]
+    def _momentum(self) -> sp.Expr:
+        return self.args[0]  # type: ignore[return-value]
 
     def evaluate(self) -> sp.Expr:
         p = self._momentum
         eta = MinkowskiMetric(p)
         return ArrayMultiplication(eta, p)
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self._momentum)
         return Rf"-\left({momentum}\right)"
 
@@ -459,16 +431,14 @@ class MinkowskiMetric(NumPyPrintable):
     # pylint: disable=no-self-use
     r"""Minkowski metric :math:`\eta = (1, -1, -1, -1)`."""
 
-    def __new__(
-        cls, momentum: FourMomentumSymbol, **hints: Any
-    ) -> "MinkowskiMetric":
+    def __new__(cls, momentum: sp.Basic, **hints) -> MinkowskiMetric:
         return create_expression(cls, momentum, **hints)
 
     @property
-    def _momentum(self) -> "MinkowskiMetric":
-        return self.args[0]
+    def _momentum(self) -> MinkowskiMetric:
+        return self.args[0]  # type: ignore[return-value]
 
-    def as_explicit(self) -> sp.Expr:
+    def as_explicit(self) -> sp.MutableDenseMatrix:
         return sp.Matrix(
             [
                 [1, 0, 0, 0],
@@ -478,10 +448,10 @@ class MinkowskiMetric(NumPyPrintable):
             ]
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         return R"\boldsymbol{\eta}"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         printer.module_imports[printer._module].update(
             {"array", "ones", "zeros"}
         )
@@ -510,15 +480,15 @@ class BoostZMatrix(UnevaluatedExpression):
     """
 
     def __new__(
-        cls, beta: sp.Expr, n_events: Optional[sp.Symbol] = None, **kwargs: Any
-    ) -> "BoostZMatrix":
+        cls, beta: sp.Basic, n_events: sp.Expr | None = None, **kwargs
+    ) -> BoostZMatrix:
         if n_events is None:
             n_events = _ArraySize(beta)
         return create_expression(cls, beta, n_events, **kwargs)
 
-    def as_explicit(self) -> sp.Expr:
+    def as_explicit(self) -> sp.MutableDenseMatrix:
         beta = self.args[0]
-        gamma = 1 / ComplexSqrt(1 - beta**2)
+        gamma = 1 / ComplexSqrt(1 - beta**2)  # type: ignore[operator]
         return sp.Matrix(
             [
                 [gamma, 0, 0, -gamma * beta],
@@ -528,9 +498,9 @@ class BoostZMatrix(UnevaluatedExpression):
             ]
         )
 
-    def evaluate(self) -> "_BoostZMatrixImplementation":
+    def evaluate(self) -> _BoostZMatrixImplementation:
         beta = self.args[0]
-        gamma = 1 / sp.sqrt(1 - beta**2)
+        gamma = 1 / sp.sqrt(1 - beta**2)  # type: ignore[operator]
         n_events = self.args[1]
         return _BoostZMatrixImplementation(
             beta=beta,
@@ -540,29 +510,29 @@ class BoostZMatrix(UnevaluatedExpression):
             zeros=_ZerosArray(n_events),
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         return printer._print(self.evaluate(), *args)
 
 
 class _BoostZMatrixImplementation(NumPyPrintable):
     def __new__(  # pylint: disable=too-many-arguments
         cls,
-        beta: sp.Expr,
-        gamma: sp.Expr,
-        gamma_beta: sp.Expr,
-        ones: "_OnesArray",
-        zeros: "_ZerosArray",
-        **hints: Any,
-    ) -> "_BoostZMatrixImplementation":
+        beta: sp.Basic,
+        gamma: sp.Basic,
+        gamma_beta: sp.Basic,
+        ones: _OnesArray,
+        zeros: _ZerosArray,
+        **hints,
+    ) -> _BoostZMatrixImplementation:
         return create_expression(
             cls, beta, gamma, gamma_beta, ones, zeros, **hints
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         beta = printer._print(self.args[0])
         return Rf"\boldsymbol{{B_z}}\left({beta}\right)"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         printer.module_imports[printer._module].add("array")
         _, gamma, gamma_beta, ones, zeros = map(printer._print, self.args)
         return f"""array(
@@ -579,12 +549,10 @@ class _BoostZMatrixImplementation(NumPyPrintable):
 class BoostMatrix(UnevaluatedExpression):
     r"""Compute a rank-3 Lorentz boost matrix from a `FourMomentumSymbol`."""
 
-    def __new__(
-        cls, momentum: FourMomentumSymbol, **kwargs: Any
-    ) -> "BoostMatrix":
+    def __new__(cls, momentum: sp.Basic, **kwargs) -> BoostMatrix:
         return create_expression(cls, momentum, **kwargs)
 
-    def as_explicit(self) -> sp.Expr:
+    def as_explicit(self) -> sp.MutableDenseMatrix:
         momentum = self.args[0]
         energy = Energy(momentum)
         beta_sq = EuclideanNormSquared(ThreeMomentum(momentum)) / energy**2
@@ -616,7 +584,7 @@ class BoostMatrix(UnevaluatedExpression):
             ]
         )
 
-    def evaluate(self) -> "_BoostMatrixImplementation":
+    def evaluate(self) -> _BoostMatrixImplementation:
         momentum = self.args[0]
         energy = Energy(momentum)
         beta_sq = EuclideanNormSquared(ThreeMomentum(momentum)) / energy**2
@@ -638,7 +606,7 @@ class BoostMatrix(UnevaluatedExpression):
             b33=1 + (gamma - 1) * beta_z**2 / beta_sq,
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self.args[0])
         return Rf"\boldsymbol{{B}}\left({momentum}\right)"
 
@@ -646,7 +614,7 @@ class BoostMatrix(UnevaluatedExpression):
 class _BoostMatrixImplementation(NumPyPrintable):
     def __new__(  # pylint: disable=too-many-arguments,too-many-locals
         cls,
-        momentum: FourMomentumSymbol,
+        momentum: sp.Basic,
         b00: sp.Basic,
         b01: sp.Basic,
         b02: sp.Basic,
@@ -657,8 +625,8 @@ class _BoostMatrixImplementation(NumPyPrintable):
         b22: sp.Basic,
         b23: sp.Basic,
         b33: sp.Basic,
-        **kwargs: Any,
-    ) -> "BoostZMatrix":
+        **kwargs,
+    ) -> _BoostMatrixImplementation:
         return create_expression(
             cls,
             momentum,
@@ -675,11 +643,11 @@ class _BoostMatrixImplementation(NumPyPrintable):
             **kwargs,
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         momentum = printer._print(self.args[0])
         return Rf"\boldsymbol{{B}}\left({momentum}\right)"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         # pylint: disable=too-many-locals, unbalanced-tuple-unpacking
         _, b00, b01, b02, b03, b11, b12, b13, b22, b23, b33 = self.args
         return f"""array(
@@ -703,13 +671,13 @@ class RotationYMatrix(UnevaluatedExpression):
     """
 
     def __new__(
-        cls, angle: sp.Expr, n_events: Optional[sp.Symbol] = None, **hints: Any
-    ) -> "RotationYMatrix":
+        cls, angle: sp.Basic, n_events: sp.Expr | None = None, **hints
+    ) -> RotationYMatrix:
         if n_events is None:
             n_events = _ArraySize(angle)
         return create_expression(cls, angle, n_events, **hints)
 
-    def as_explicit(self) -> sp.Expr:
+    def as_explicit(self) -> sp.MutableDenseMatrix:
         angle = self.args[0]
         return sp.Matrix(
             [
@@ -720,7 +688,7 @@ class RotationYMatrix(UnevaluatedExpression):
             ]
         )
 
-    def evaluate(self) -> "_RotationYMatrixImplementation":
+    def evaluate(self) -> _RotationYMatrixImplementation:
         angle = self.args[0]
         n_events = self.args[1]
         return _RotationYMatrixImplementation(
@@ -731,30 +699,30 @@ class RotationYMatrix(UnevaluatedExpression):
             zeros=_ZerosArray(n_events),
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         return printer._print(self.evaluate(), *args)
 
 
 class _RotationYMatrixImplementation(NumPyPrintable):
     def __new__(  # pylint: disable=too-many-arguments
         cls,
-        angle: sp.Expr,
-        cos_angle: sp.Expr,
-        sin_angle: sp.Expr,
-        ones: "_OnesArray",
-        zeros: "_ZerosArray",
-        **hints: Any,
-    ) -> "_RotationYMatrixImplementation":
+        angle: sp.Basic,
+        cos_angle: sp.Basic,
+        sin_angle: sp.Basic,
+        ones: _OnesArray,
+        zeros: _ZerosArray,
+        **hints,
+    ) -> _RotationYMatrixImplementation:
         return create_expression(
             cls, angle, cos_angle, sin_angle, ones, zeros, **hints
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         angle, *_ = self.args
         angle = printer._print(angle)
         return Rf"\boldsymbol{{R_y}}\left({angle}\right)"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         printer.module_imports[printer._module].add("array")
         _, cos_angle, sin_angle, ones, zeros = map(printer._print, self.args)
         return f"""array(
@@ -778,13 +746,13 @@ class RotationZMatrix(UnevaluatedExpression):
     """
 
     def __new__(
-        cls, angle: sp.Expr, n_events: Optional[sp.Symbol] = None, **hints: Any
-    ) -> "RotationZMatrix":
+        cls, angle: sp.Basic, n_events: sp.Expr | None = None, **hints
+    ) -> RotationZMatrix:
         if n_events is None:
             n_events = _ArraySize(angle)
         return create_expression(cls, angle, n_events, **hints)
 
-    def as_explicit(self) -> sp.Expr:
+    def as_explicit(self) -> sp.MutableDenseMatrix:
         angle = self.args[0]
         return sp.Matrix(
             [
@@ -795,7 +763,7 @@ class RotationZMatrix(UnevaluatedExpression):
             ]
         )
 
-    def evaluate(self) -> "_RotationZMatrixImplementation":
+    def evaluate(self) -> _RotationZMatrixImplementation:
         angle = self.args[0]
         n_events = self.args[1]
         return _RotationZMatrixImplementation(
@@ -806,30 +774,30 @@ class RotationZMatrix(UnevaluatedExpression):
             zeros=_ZerosArray(n_events),
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         return printer._print(self.evaluate(), *args)
 
 
 class _RotationZMatrixImplementation(NumPyPrintable):
     def __new__(  # pylint: disable=too-many-arguments
         cls,
-        angle: sp.Expr,
-        cos_angle: sp.Expr,
-        sin_angle: sp.Expr,
-        ones: "_OnesArray",
-        zeros: "_ZerosArray",
-        **hints: Any,
-    ) -> "_RotationZMatrixImplementation":
+        angle: sp.Basic,
+        cos_angle: sp.Basic,
+        sin_angle: sp.Basic,
+        ones: _OnesArray,
+        zeros: _ZerosArray,
+        **hints,
+    ) -> _RotationZMatrixImplementation:
         return create_expression(
             cls, angle, cos_angle, sin_angle, ones, zeros, **hints
         )
 
-    def _latex(self, printer: LatexPrinter, *args: Any) -> str:
+    def _latex(self, printer: LatexPrinter, *args) -> str:
         angle, *_ = self.args
         angle = printer._print(angle)
         return Rf"\boldsymbol{{R_z}}\left({angle}\right)"
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         printer.module_imports[printer._module].add("array")
         _, cos_angle, sin_angle, ones, zeros = map(printer._print, self.args)
         return f"""array(
@@ -843,41 +811,37 @@ class _RotationZMatrixImplementation(NumPyPrintable):
 
 
 class _OnesArray(NumPyPrintable):
-    def __new__(
-        cls, shape: Union[int, Sequence[int]], **kwargs: Any
-    ) -> "_OnesArray":
+    def __new__(cls, shape, **kwargs) -> _OnesArray:
         return create_expression(cls, shape, **kwargs)
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         printer.module_imports[printer._module].add("ones")
         shape = printer._print(self.args[0])
         return f"ones({shape})"
 
 
 class _ZerosArray(NumPyPrintable):
-    def __new__(
-        cls, shape: Union[int, Sequence[int]], **kwargs: Any
-    ) -> "_ZerosArray":
+    def __new__(cls, shape, **kwargs) -> _ZerosArray:
         return create_expression(cls, shape, **kwargs)
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         printer.module_imports[printer._module].add("zeros")
         shape = printer._print(self.args[0])
         return f"zeros({shape})"
 
 
 class _ArraySize(NumPyPrintable):
-    def __new__(cls, array: sp.Basic, **kwargs: Any) -> "_ArraySize":
+    def __new__(cls, array: sp.Basic, **kwargs) -> _ArraySize:
         return create_expression(cls, array, **kwargs)
 
-    def _numpycode(self, printer: NumPyPrinter, *args: Any) -> str:
+    def _numpycode(self, printer: NumPyPrinter, *args) -> str:
         shape = printer._print(self.args[0])
         return f"len({shape})"
 
 
 def compute_helicity_angles(
-    four_momenta: "FourMomenta", topology: Topology
-) -> Dict[str, sp.Expr]:
+    four_momenta: Mapping[int, sp.Expr], topology: Topology
+) -> dict[str, sp.Expr]:
     """Formulate expressions for all helicity angles in a topology.
 
     Formulate expressions (`~sympy.core.expr.Expr`) for all helicity angles
@@ -904,9 +868,9 @@ def compute_helicity_angles(
     n_events = _get_number_of_events(four_momenta)
 
     def __recursive_helicity_angles(  # pylint: disable=too-many-locals
-        four_momenta: "FourMomenta", node_id: int
-    ) -> Dict[str, sp.Expr]:
-        helicity_angles: Dict[str, sp.Expr] = {}
+        four_momenta: Mapping[int, sp.Expr], node_id: int
+    ) -> dict[str, sp.Expr]:
+        helicity_angles: dict[str, sp.Expr] = {}
         child_state_ids = sorted(
             topology.get_edge_ids_outgoing_from_node(node_id)
         )
@@ -916,7 +880,7 @@ def compute_helicity_angles(
             state_id = child_state_ids[0]
             if is_opposite_helicity_state(topology, state_id):
                 state_id = child_state_ids[1]
-            four_momentum = four_momenta[state_id]
+            four_momentum: sp.Expr = four_momenta[state_id]
             phi_label, theta_label = get_helicity_angle_label(
                 topology, state_id
             )
@@ -940,7 +904,7 @@ def compute_helicity_angles(
                     theta = Theta(four_momentum)
                     p3_norm = three_momentum_norm(four_momentum)
                     beta = p3_norm / Energy(four_momentum)
-                    new_momentum_pool = {
+                    new_momentum_pool: dict[int, sp.Expr] = {
                         k: ArrayMultiplication(
                             BoostZMatrix(beta, n_events),
                             RotationYMatrix(-theta, n_events),
@@ -978,22 +942,22 @@ def compute_helicity_angles(
 
 
 def _get_number_of_events(
-    four_momenta: "FourMomenta",
-) -> "_ArraySize":
+    four_momenta: Mapping[int, sp.Expr],
+) -> _ArraySize:
     sorted_momentum_symbols = sorted(four_momenta.values(), key=str)
     return _ArraySize(sorted_momentum_symbols[0])
 
 
 def compute_invariant_masses(
-    four_momenta: "FourMomenta", topology: Topology
-) -> Dict[str, sp.Expr]:
+    four_momenta: FourMomenta, topology: Topology
+) -> dict[str, sp.Expr]:
     """Compute the invariant masses for all final state combinations."""
     if topology.outgoing_edge_ids != set(four_momenta):
         raise ValueError(
             f"Momentum IDs {set(four_momenta)} do not match "
             f"final state edge IDs {set(topology.outgoing_edge_ids)}"
         )
-    invariant_masses = {}
+    invariant_masses: dict[str, sp.Expr] = {}
     for state_id in topology.edges:
         attached_state_ids = determine_attached_final_state(topology, state_id)
         total_momentum = ArraySum(
@@ -1006,8 +970,8 @@ def compute_invariant_masses(
 
 
 def compute_wigner_angles(
-    topology: Topology, momenta: "FourMomenta", state_id: int
-) -> Dict[str, sp.Expr]:
+    topology: Topology, momenta: FourMomenta, state_id: int
+) -> dict[str, sp.Expr]:
     """Create an `~sympy.core.expr.Expr` for each angle in a Wigner rotation.
 
     Implementation of (B.2-4) in
@@ -1034,7 +998,7 @@ def compute_wigner_angles(
 
 
 def compute_wigner_rotation_matrix(
-    topology: Topology, momenta: "FourMomenta", state_id: int
+    topology: Topology, momenta: FourMomenta, state_id: int
 ) -> MatrixMultiplication:
     """Compute a Wigner rotation matrix.
 
@@ -1048,11 +1012,11 @@ def compute_wigner_rotation_matrix(
 
 
 def compute_boost_chain(
-    topology: Topology, momenta: "FourMomenta", state_id: int
-) -> List[BoostMatrix]:
+    topology: Topology, momenta: FourMomenta, state_id: int
+) -> list[BoostMatrix]:
     boost_matrices = []
     decay_chain_state_ids = __get_boost_chain_ids(topology, state_id)
-    boosted_momenta = {
+    boosted_momenta: dict[int, sp.Expr] = {
         i: get_four_momentum_sum(topology, momenta, i)
         for i in decay_chain_state_ids
     }
@@ -1067,7 +1031,7 @@ def compute_boost_chain(
     return boost_matrices
 
 
-def __get_boost_chain_ids(topology: Topology, state_id: int) -> List[int]:
+def __get_boost_chain_ids(topology: Topology, state_id: int) -> list[int]:
     """Get the state IDs from first resonance to this final state.
 
     >>> from qrules.topology import create_isobar_topologies
@@ -1088,8 +1052,8 @@ def __get_boost_chain_ids(topology: Topology, state_id: int) -> List[int]:
 
 
 def get_four_momentum_sum(
-    topology: Topology, momenta: "FourMomenta", state_id: int
-) -> Union[ArraySum, FourMomentumSymbol]:
+    topology: Topology, momenta: FourMomenta, state_id: int
+) -> ArraySum | FourMomentumSymbol:
     """Get the `FourMomentumSymbol` or sum of momenta for **any** edge ID.
 
     If the edge ID is a final state ID, return its `FourMomentumSymbol`. If
