@@ -13,6 +13,7 @@ The package also provides other helpful functions, like
 :func:`substitute_indexed_symbols`, that are useful when visualizing
 `sympy` expressions.
 """
+from __future__ import annotations
 
 import inspect
 import logging
@@ -20,14 +21,11 @@ import sys
 from collections import abc
 from typing import (
     TYPE_CHECKING,
-    Any,
     Callable,
-    Dict,
     Iterator,
     Mapping,
     Sequence,
     Tuple,
-    Type,
     TypeVar,
     Union,
 )
@@ -36,12 +34,9 @@ import sympy as sp
 from ipywidgets.widgets import FloatSlider, IntSlider
 from sympy.printing.latex import translate
 
-try:
-    from IPython.lib.pretty import PrettyPrinter
-except ImportError:
-    PrettyPrinter = Any
-
 if TYPE_CHECKING:  # pragma: no cover
+    from IPython.lib.pretty import PrettyPrinter
+
     if sys.version_info >= (3, 10):
         from typing import TypeGuard  # pylint: disable=no-name-in-module
     else:
@@ -81,12 +76,12 @@ class SliderKwargs(abc.Mapping):
         }
 
     @property
-    def arg_to_symbol(self) -> Dict[str, str]:
+    def arg_to_symbol(self) -> dict[str, str]:
         """**Copy** of the internal translation `dict` for argument names."""
         return dict(self._arg_to_symbol)
 
     @property
-    def symbol_to_arg(self) -> Dict[str, str]:
+    def symbol_to_arg(self) -> dict[str, str]:
         """Inverted `dict` of `arg_to_symbol`."""
         return {symbol: arg for arg, symbol in self._arg_to_symbol.items()}
 
@@ -117,7 +112,7 @@ class SliderKwargs(abc.Mapping):
                     f'Slider "{name}" is not a valid ipywidgets slider'
                 )
 
-    def __getitem__(self, key: Union[str, sp.Symbol]) -> "Slider":
+    def __getitem__(self, key: str | sp.Symbol) -> Slider:
         """Get slider by symbol, symbol name, or argument name."""
         if isinstance(key, sp.Symbol):
             key = key.name
@@ -164,7 +159,7 @@ class SliderKwargs(abc.Mapping):
             p.breakable()
             p.text(")")
 
-    def set_values(self, *args: Dict[str, float], **kwargs: float) -> None:
+    def set_values(self, *args: dict[str, float], **kwargs: float) -> None:
         """Set initial values for the sliders.
 
         Either use a `dict` as input, or use :term:`kwargs <python:keyword
@@ -183,7 +178,7 @@ class SliderKwargs(abc.Mapping):
                 continue
 
     def set_ranges(  # noqa: R701
-        self, *args: Dict[str, RangeDefinition], **kwargs: "RangeDefinition"
+        self, *args: dict[str, RangeDefinition], **kwargs: RangeDefinition
     ) -> None:
         """Set min, max and (optionally) the nr of steps for each slider.
 
@@ -227,7 +222,7 @@ class SliderKwargs(abc.Mapping):
 
 def _is_min_max(
     range_def: RangeDefinition,
-) -> "TypeGuard[Tuple[float, float]]":
+) -> TypeGuard[tuple[float, float]]:
     if len(range_def) == 2:
         return True
     return False
@@ -235,7 +230,7 @@ def _is_min_max(
 
 def _is_min_max_step(
     range_def: RangeDefinition,
-) -> "TypeGuard[Tuple[float, float, Union[float, int]]]":
+) -> TypeGuard[tuple[float, float, float | int]]:
     if len(range_def) == 3:
         return True
     return False
@@ -245,8 +240,8 @@ ValueType = TypeVar("ValueType")
 
 
 def _merge_args_kwargs(
-    *args: Dict[str, ValueType], **kwargs: ValueType
-) -> Dict[str, ValueType]:
+    *args: dict[str, ValueType], **kwargs: ValueType
+) -> dict[str, ValueType]:
     r"""Merge positional `dict` arguments and keyword arguments into one `dict`.
 
     >>> _merge_args_kwargs(x="X", y="Y")
@@ -264,8 +259,8 @@ def _merge_args_kwargs(
 
 
 def prepare_sliders(
-    expression: sp.Expr, plot_symbol: Union[sp.Symbol, Tuple[sp.Symbol, ...]]
-) -> Tuple[Callable, SliderKwargs]:
+    expression: sp.Expr, plot_symbol: sp.Symbol | tuple[sp.Symbol, ...]
+) -> tuple[Callable, SliderKwargs]:
     # cspell:ignore lambdifygenerated
     """Lambdify a `sympy` expression and create sliders for its arguments.
 
@@ -294,7 +289,7 @@ def prepare_sliders(
     return lambdified_expression, sliders
 
 
-def create_slider(symbol: sp.Symbol) -> "Slider":
+def create_slider(symbol: sp.Symbol) -> Slider:
     r"""Create an `int` or `float` slider, depending on Symbol assumptions.
 
     The description for the slider is rendered as LaTeX from the
@@ -306,18 +301,18 @@ def create_slider(symbol: sp.Symbol) -> "Slider":
     IntSlider(value=0, description='\\(n_{0}\\)')
     """
     description = Rf"\({sp.latex(symbol)}\)"
-    if symbol.is_integer:
+    if symbol.is_integer:  # type: ignore[attr-defined]
         return IntSlider(description=description)
     return FloatSlider(description=description)
 
 
 def _extract_slider_symbols(
     expression: sp.Expr,
-    plot_symbol: Union[sp.Symbol, Sequence[sp.Symbol]],
-) -> Tuple[sp.Symbol, ...]:
+    plot_symbol: sp.Symbol | Sequence[sp.Symbol],
+) -> tuple[sp.Symbol, ...]:
     """Extract sorted, remaining free symbols of a `sympy` expression."""
     plot_symbols = __safe_wrap_symbols(plot_symbol)
-    free_symbols = set(expression.free_symbols)
+    free_symbols: set[sp.Symbol] = expression.free_symbols  # type: ignore[assignment]
     for symbol in plot_symbols:
         if symbol not in free_symbols:
             raise ValueError(
@@ -329,8 +324,8 @@ def _extract_slider_symbols(
 
 
 def __safe_wrap_symbols(
-    plot_symbol: Union[sp.Symbol, Sequence[sp.Symbol]]
-) -> Tuple[sp.Symbol, ...]:
+    plot_symbol: sp.Symbol | Sequence[sp.Symbol],
+) -> tuple[sp.Symbol, ...]:
     if isinstance(plot_symbol, abc.Sequence):
         return tuple(plot_symbol)
     if isinstance(plot_symbol, sp.Symbol):
@@ -342,7 +337,7 @@ def __safe_wrap_symbols(
 
 def partial_doit(
     expression: sp.Expr,
-    doit_classes: Union[Type[sp.Basic], Tuple[Type[sp.Basic], ...]],
+    doit_classes: type[sp.Basic] | tuple[type[sp.Basic], ...],
 ) -> sp.Expr:
     """Perform :meth:`~sympy.core.basic.Basic.doit` up to a certain level.
 
@@ -372,7 +367,7 @@ def _indexed_to_symbol(idx: sp.Indexed) -> sp.Symbol:
 
 
 def rename_symbols(
-    expression: sp.Expr, renames: Union[Callable[[str], str], Dict[str, str]]
+    expression: sp.Expr, renames: Callable[[str], str] | dict[str, str]
 ) -> sp.Expr:
     r"""Rename symbols in an expression.
 
@@ -387,18 +382,17 @@ def rename_symbols(
         ...
     KeyError: "No symbol with name 'non-existent' in expression"
     """
-    substitutions: Dict[sp.Symbol, sp.Symbol] = {}
+    substitutions: dict[sp.Symbol, sp.Symbol] = {}
+    free_symbols: set[sp.Symbol] = expression.free_symbols  # type: ignore[assignment]
     if callable(renames):
-        for old_symbol in expression.free_symbols:
+        for old_symbol in free_symbols:
             new_name = renames(old_symbol.name)
             new_symbol = sp.Symbol(new_name, **old_symbol.assumptions0)
             substitutions[old_symbol] = new_symbol
     elif isinstance(renames, dict):
         for old_name, new_name in renames.items():
             # pylint: disable=cell-var-from-loop
-            matches = filter(
-                lambda s: s.name == old_name, expression.free_symbols
-            )
+            matches = filter(lambda s: s.name == old_name, free_symbols)
             try:
                 old_symbol = next(matches)
             except StopIteration:
