@@ -118,6 +118,55 @@ class TestHelicityAmplitudeBuilder:
         assert initial_state_mass not in model.kinematic_variables
         assert initial_state_mass in model.parameter_defaults
 
+    def test_use_helicity_couplings(self, reaction: ReactionInfo):
+        # cspell:ignore coeff
+        builder: HelicityAmplitudeBuilder = get_builder(reaction)
+        builder.use_helicity_couplings = False
+        coeff_model = builder.formulate()
+        builder.use_helicity_couplings = True
+        coupling_model = builder.formulate()
+
+        coefficient_names = {p.name for p in coeff_model.parameter_defaults}
+        coupling_names = {p.name for p in coupling_model.parameter_defaults}
+        if reaction.formalism == "canonical-helicity":
+            assert len(coefficient_names) == 4
+            assert coefficient_names == {
+                R"C_{J/\psi(1S) \xrightarrow[S=1]{L=0} f_{0}(980) \gamma;"
+                R" f_{0}(980) \xrightarrow[S=0]{L=0} \pi^{0} \pi^{0}}",
+                R"C_{J/\psi(1S) \xrightarrow[S=1]{L=2} f_{0}(1500) \gamma;"
+                R" f_{0}(1500) \xrightarrow[S=0]{L=0} \pi^{0} \pi^{0}}",
+                R"C_{J/\psi(1S) \xrightarrow[S=1]{L=0} f_{0}(1500) \gamma;"
+                R" f_{0}(1500) \xrightarrow[S=0]{L=0} \pi^{0} \pi^{0}}",
+                R"C_{J/\psi(1S) \xrightarrow[S=1]{L=2} f_{0}(980) \gamma;"
+                R" f_{0}(980) \xrightarrow[S=0]{L=0} \pi^{0} \pi^{0}}",
+            }
+            assert len(coupling_names) == 6
+            assert coupling_names == {
+                R"H_{J/\psi(1S) \xrightarrow[S=1]{L=0} f_{0}(1500) \gamma}",
+                R"H_{J/\psi(1S) \xrightarrow[S=1]{L=0} f_{0}(980) \gamma}",
+                R"H_{J/\psi(1S) \xrightarrow[S=1]{L=2} f_{0}(1500) \gamma}",
+                R"H_{J/\psi(1S) \xrightarrow[S=1]{L=2} f_{0}(980) \gamma}",
+                R"H_{f_{0}(1500) \xrightarrow[S=0]{L=0} \pi^{0} \pi^{0}}",
+                R"H_{f_{0}(980) \xrightarrow[S=0]{L=0} \pi^{0} \pi^{0}}",
+            }
+        else:
+            assert len(coefficient_names) == 2
+            assert coefficient_names == {
+                R"C_{J/\psi(1S) \to {f_{0}(980)}_{0} \gamma_{+1}; f_{0}(980)"
+                R" \to \pi^{0}_{0} \pi^{0}_{0}}",
+                R"C_{J/\psi(1S) \to {f_{0}(1500)}_{0} \gamma_{+1};"
+                R" f_{0}(1500) \to \pi^{0}_{0} \pi^{0}_{0}}",
+            }
+            assert len(coupling_names) == 6
+            assert coupling_names == {
+                R"H_{J/\psi(1S) \to {f_{0}(1500)}_{0} \gamma_{+1}}",
+                R"H_{J/\psi(1S) \to {f_{0}(1500)}_{0} \gamma_{-1}}",
+                R"H_{J/\psi(1S) \to {f_{0}(980)}_{0} \gamma_{+1}}",
+                R"H_{J/\psi(1S) \to {f_{0}(980)}_{0} \gamma_{-1}}",
+                R"H_{f_{0}(1500) \to \pi^{0}_{0} \pi^{0}_{0}}",
+                R"H_{f_{0}(980) \to \pi^{0}_{0} \pi^{0}_{0}}",
+            }
+
 
 class TestHelicityModel:
     def test_rename_symbols_no_renames(
