@@ -1,5 +1,7 @@
-# pylint: disable=redefined-outer-name,no-self-use
-from typing import Any, List, Optional, Sequence, Union
+# pylint: disable=no-self-use, redefined-outer-name
+from __future__ import annotations
+
+from typing import Iterable, Sequence
 
 import pytest
 import qrules
@@ -11,9 +13,9 @@ from ampform import get_builder
 
 
 def calculate_sympy_integral(
-    intensity: Any,
-    integration_variables: List[sp.Symbol],
-    jacobi_determinant: Optional[Any] = None,
+    intensity,
+    integration_variables: Iterable[sp.Symbol],
+    jacobi_determinant=None,
 ) -> sp.Expr:
     if jacobi_determinant is None:
         for int_var in integration_variables:
@@ -112,11 +114,12 @@ class TestEpemToDmD0Pip:
     )
     def test_angular_distributions(
         self,
-        angular_variables: Union[str, Sequence[str]],
+        angular_variables: str | Sequence[str],
         expected_distribution_function: sp.Expr,
         sympy_model: sp.Expr,
     ) -> None:
-        assert {s.name for s in sympy_model.free_symbols} == {
+        free_symbols: set[sp.Symbol] = sympy_model.free_symbols  # type: ignore[assignment]
+        assert {s.name for s in free_symbols} == {
             "phi_0^02",
             "theta_02",
             "theta_0^02",
@@ -128,9 +131,7 @@ class TestEpemToDmD0Pip:
         # remove angular variable
         integration_variable_set = set(angular_variables)
         integration_variables = [
-            x
-            for x in sympy_model.free_symbols
-            if x.name not in integration_variable_set
+            x for x in free_symbols if x.name not in integration_variable_set
         ]
 
         # Note: using nsimplify with rational=True solves assertion failure due
@@ -203,11 +204,12 @@ class TestD1ToD0PiPi:
     )
     def test_angular_distributions(
         self,
-        angular_variables: Union[str, Sequence[str]],
+        angular_variables: str | Sequence[str],
         expected_distribution_function: sp.Expr,
         sympy_model: sp.Expr,
     ) -> None:
-        assert {s.name for s in sympy_model.free_symbols} == {
+        free_symbols: set[sp.Symbol] = sympy_model.free_symbols  # type: ignore[assignment]
+        assert {s.name for s in free_symbols} == {
             "phi_0^02",
             "theta_02",
             "theta_0^02",
@@ -219,9 +221,7 @@ class TestD1ToD0PiPi:
         # remove angular variable
         integration_variable_set = set(angular_variables)
         integration_variables = [
-            x
-            for x in sympy_model.free_symbols
-            if x.name not in integration_variable_set
+            x for x in free_symbols if x.name not in integration_variable_set
         ]
 
         # Note: using nsimplify with rational=True solves assertion failure due
